@@ -1454,6 +1454,8 @@ static int initiate_scan(struct supplicant_task *task)
 	if (task->scan_call != NULL)
 		return -EALREADY;
 
+	connman_info("%s initiating scan", task->ifname);
+
 	message = dbus_message_new_method_call(SUPPLICANT_NAME, task->path,
 					SUPPLICANT_INTF ".Interface", "scan");
 	if (message == NULL)
@@ -1927,9 +1929,6 @@ static void properties_reply(DBusPendingCall *call, void *user_data)
 		index = connman_device_get_index(task->device);
 		connman_network_set_index(network, index);
 
-		connman_network_set_protocol(network,
-						CONNMAN_NETWORK_PROTOCOL_IP);
-
 		connman_network_set_address(network, result.addr,
 							result.addr_len);
 
@@ -2536,6 +2535,9 @@ int supplicant_disconnect(struct connman_network *network)
 	if (task->disconnecting == TRUE)
 		return -EALREADY;
 
+	if (task->network != network)
+		return -EINVAL;
+
 	remove_network(task);
 
 	disconnect_network(task);
