@@ -185,6 +185,19 @@ void connman_dhcp_renew(struct connman_dhcp *dhcp)
 	connman_element_update(dhcp->element);
 }
 
+/**
+ * connman_dhcp_release:
+ * @dhcp: DHCP structure
+ *
+ * Report DHCP release of the interface
+ */
+void connman_dhcp_release(struct connman_dhcp *dhcp)
+{
+	DBG("dhcp %p", dhcp);
+
+	connman_element_unregister_children(dhcp->element);
+}
+
 /**
  * connman_dhcp_fail:
  * @dhcp: DHCP structure
